import QtQuick 2.4
import QtQuick.Controls 1.0
import QtWebEngine 1.0
import "."

Item {
    id: root

    property string url: defaultURL
    property bool keyingURL: false
    property string defaultURL: application.homeUrl
    property bool noSwipe: true
    property int currentContentWidth:0

    Connections {
        target: application

        onCommandReceived: {
            processCommand(cmd)
        }
    }

    function processCommand(cmd) {
        var parts = cmd.split(" ")
        if (parts.length < 2) {
            console.log("invalid command", cmd)
            return
        }

        switch (parts[1]) {
            case "back":
                webView.goBack()
                break
            case "forward":
                webView.goForward()
                break
            case "input":
                break
            case "url":
                var url = parts.slice(2).join(" ") // rejoin parts
                root.url = url
                break
            case "clearCookies":
                console.log("TODO clear cookies")
                break
            case "hidden":
//                webView.renderingEnabled = false
                break
            case "shown":
//                webView.renderingEnabled = true
                break
            default:
                console.log("unknown command", parts[1])
        }
    }

    function clearCookies() {
        nami.clearCookies()
        //webView.reload.trigger()
    }

    onUrlChanged: {
        webView.url = webView.fixUrl(url)
    }

    BrowserTerms {
        id: webTermsPrompt

        Component.onCompleted: hide()
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.horizontalCenterOffset: 20
        anchors.verticalCenter: parent.verticalCenter

        onYesClicked: {
            hide()
        }

        onNoClicked: {
        }
    }

    Rectangle {
        id: buttonBar

        color: "darkgray"
        radius: 5
        anchors { top: parent.top ; left: parent.left ; right: parent.right }
        height: 40

        Row {
            id: buttonRow
            spacing: 15

            ImageButton {
                id: backButton

                upimage: "graphics/web_leftarrow.png"
                downimage: "graphics/web_leftarrow_DOWN.png"
//                enabled: webView.back.enabled//TODO
                anchors { top: parent.top ; topMargin: 2}

                onClicked: webView.goBack()
            }

            ImageButton {
                id: forwardButton

                upimage: "graphics/web_rightarrow.png"
                downimage: "graphics/web_rightarrow_DOWN.png"
//                enabled: webView.forward.enabled//TODO
                anchors { top: parent.top ; topMargin: 2 }

                onClicked: webView.goForward()
            }

            ImageButton {
                id: fullscreenButton

                visible: true
                upimage: "graphics/web_fullscreen.png"
                downimage: "graphics/web_fullscreen_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }
                onClicked: {
                    console.log("CMD fullscreen\n")
                }
            }

            ImageButton {
                id: keyboardButton

                upimage: "graphics/web_keyboard.png"
                downimage: "graphics/web_keyboard_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }

                onClicked: {
                    if (keypadManager.checkVisible("inputKeypad")) {
                        keypadManager.hideKeypad("inputKeypad")
                    } else {
                        keypadManager.hideKeypad("urlKeypad")
                        keypadManager.showKeypad("inputKeypad")
                    }
                }
            }

            ImageButton {
                id: homeButton

                visible: application.homeUrl
                upimage: "graphics/web_home.png"
                downimage: "graphics/web_home_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }
                onClicked: root.url = root.defaultURL
            }

            ImageButton {
                id: linksButton

                visible: application.linksUrl
                upimage: "graphics/web_bookmarks.png"
                downimage: "graphics/web_bookmarks_DOWN.png"
                anchors { top: parent.top ; topMargin: 2 }
                onClicked: root.url = application.linksUrl
            }

        }
        Rectangle {
            id: urlBar

            color: "white"
            radius: 5
            height: 32
            anchors { left: buttonRow.right ; leftMargin: 15 ;  right: parent.right ; rightMargin: 8 ; top: parent.top; topMargin: 3 }

            Rectangle {
                id: progressBar

                color: "lightblue"
                opacity: 0.6
                radius: 5
                width: (webView.progress < 1) ? webView.progress* urlBar.width : 0
                anchors { left: urlBar.left ; top: urlBar.top ; bottom: urlBar.bottom }
            }

            Flickable {
                id: textFlickable
                anchors { verticalCenter: parent.verticalCenter ; left: parent.left ; leftMargin: 5 }
                clip: true
                width: parent.width - 60
                height: urlEntry.height
                flickableDirection: Flickable.HorizontalFlick
                boundsBehavior: Flickable.DragOverBounds
                interactive: contentWidth - 4 > width
                contentWidth: urlEntry.width+2
                contentHeight: urlEntry.height

                TextInput {
                    id: urlEntry

                    function adjustTextPosition() {
                        var offset = -1
                        var rect = urlEntry.positionToRectangle(cursorPosition)
                        if (rect.x - textFlickable.contentX + 30 > textFlickable.width) {
                            offset = Math.max(0, rect.x - textFlickable.width)
                            // show some trailing characters if not at the end
                            if (cursorPosition < urlEntry.text.length - 1)
                                offset += 30
                        }
                        if (offset >= 0)
                            textFlickable.contentX = offset
                    }

                    width: Math.max(textFlickable.width, implicitWidth)
                    Connections {
                        target: webView
                        onUrlChanged: {
                            urlEntry.text = webView.url
                            urlEntry.cursorPosition = 0
                            textFlickable.contentX = 0
                        }
                    }
                    font.pointSize: 12
                    onActiveFocusChanged: {
                        if (activeFocus) {
                            keypadManager.hideKeypad("inputKeypad")
                            keypadManager.showKeypad("urlKeypad")
                        } else {
                            keypadManager.hideKeypad("urlKeypad")
                            urlEntry.cursorPosition = 0
                            textFlickable.contentX = 0
                        }
                    }

                    onTextChanged: {
                        urlEntry.adjustTextPosition()

                    }

                    onAccepted: {
                        root.url = urlEntry.text
                        webView.focus = true
                    }

                    //                MouseArea {
                    //                    anchors.fill: parent
                    //                    onClicked: {
                    //                        urlEntry.forceActiveFocus()
                    //                    }
                    //                }
                }
            }

            ImageButton {
                upimage: "graphics/KeyPad_X_Large.png"
                downimage: upimage
                onClicked: {
                    urlEntry.text = ""
                    urlEntry.focus = true
                    textFlickable.contentX = 0
                }
                anchors { right: parent.right; rightMargin: 0; verticalCenter: parent.verticalCenter }
            }
        }
    }

//        id: scrollView
//        anchors { top: buttonBar.bottom ; bottom: parent.bottom ; left: parent.left ; right: parent.right }
//        width: 1000
//        height: 406

    WebEngineView {
        id: webView
        focus: true
        anchors {
            top: buttonBar.bottom
            bottom: parent.bottom
            left: parent.left
            right: parent.right
        }

        //            settings.pluginsEnabled: true
        //            settings.javascriptCanOpenWindows: false
        //            settings.developerExtrasEnabled: true
        //            settings.javascriptEnabled: root.url.indexOf("facebook.com") < 0

        // TODO can we handle target=_blank type links, i.e. pages that open in new window?
        //            newWindowParent: webView
        //            newWindowComponent: Component {
        //                Item {
        //                    id: newPageComponent
        //                    WebView {
        //                        id: newWindow
        //                        onUrlChanged: {
        //                            console.log("Opening", url, "in current window rather than new window")
        //                            root.url = url

        //                            var newObject = Qt.createQmlObject("import QtQuick 1.0; Item {}", webView)
        //                            newPageComponent.parent = newObject
        //                            newObject.destroy()
        //                        }
        //                    }
        //                }
        //            }

        url: root.defaultURL

        function fixUrl(url) {
            if(url=="") return url
            if(url[0] == "/") return "file://" + url
            if(url.indexOf(":") < 0) {
                if(url.indexOf(".") < 0 || url.indexOf(" ") >= 0){
                    print("url fallback ")
                    if(root.defaultURL)
                        return root.defaultURL
                    else
                        return "http://www.truefitness.com"
                }
                else
                    return urlRewrite("http://" + url)
            }
            else {
                url = urlRewrite(url)
                return url
            }

        }

        function urlRewrite(url) {
            return url
            // TODO No longer needed with Qt5 WebKit (after removing Flickable)?
            //                var isFacebook = new RegExp(".*(www\.)?facebook\.com/?$")
            //                if (isFacebook.test(url) && url.indexOf("m.facebook.com") === -1) {
            //                    return "https://m.facebook.com"
            //                } else {
            //                    return url
            //                }
        }

        onLoadingChanged: {
            if (loadRequest.status === WebEngineView.LoadFailedStatus) {
                console.log("Failed to load webpage", root.url, webView.progress)
                loadFailedMessage.show(3)
            }
        }

        onUrlChanged: {
            if(url != root.url)
                root.url = url
        }

        transformOrigin: Item.TopLeft
        smooth: false
    }


    GrowlMessage {
        id: loadFailedMessage
        message: '<img src="graphics/red_x.png" /> ' + qsTr("The Requested Page Could Not Be Loaded") //+ application.emptyString
        anchors { top: buttonBar.bottom }
    }

//    Rectangle {
//        id: scrollPos

//        color: "gray"
//        opacity: 0.5
//        width: 10
//        height: flickable.height * ( flickable.height / webView.contentsSize.height)

//        anchors { right: flickable.right ; top: flickable.top ; topMargin: flickable.contentY * ( flickable.height / flickable.contentHeight)}
//    }

}
